@@ -31,58 +31,46 @@
 
 // Implementation for the Error class.
 
-
 #include "config.h"
 
-#include <cstdio>
 #include <cassert>
+#include <cstdio>
 #include <utility>
 
 #include "Error.h"
-#include "parser.h"
 #include "InternalErr.h"
 #include "debug.h"
+#include "parser.h"
 
 using namespace std;
 
 // Glue routines declared in Error.lex
 extern void Error_switch_to_buffer(void *new_buffer);
-extern void Error_delete_buffer(void * buffer);
+extern void Error_delete_buffer(void *buffer);
 extern void *Error_buffer(FILE *fp);
 
-//extern void Errorrestart(FILE *yyin); // defined in Error.tab.c
+// extern void Errorrestart(FILE *yyin); // defined in Error.tab.c
 extern int Errorparse(libdap::parser_arg *arg);
 
 namespace libdap {
 
-// There are two entries for 'cannot read file' because of an error made 
-// when the message was first added to this class. 
+// There are two entries for 'cannot read file' because of an error made
+// when the message was first added to this class.
 static const char *err_messages[] = {
-    "Undefined error",
-    "Unknown error",
-    "Internal error",
-    "No such file",
-    "No such variable",
-    "Malformed expression",
-    "No authorization",
-    "Cannot read file",
-    "Not Implemented",
-    ""
-};
+    "Undefined error",      "Unknown error",    "Internal error",   "No such file",    "No such variable",
+    "Malformed expression", "No authorization", "Cannot read file", "Not Implemented", ""};
 
-Error &
-Error::operator=(const Error &rhs)
-{
+Error &Error::operator=(const Error &rhs) {
     assert(OK());
 
-    if (&rhs == this)  // are they identical?
+    if (&rhs == this) // are they identical?
         return *this;
     else {
         _error_code = rhs._error_code;
         _error_message = rhs._error_message;
 
         d_file = rhs.d_file;
-        d_line  = rhs.d_line;
+        d_line = rhs.d_line;
 
         assert(this->OK());
 
@@ -96,16 +84,12 @@ Error::operator=(const Error &rhs)
 
     @brief Is the Error object valid?
     @return TRUE if the object is valid, FALSE otherwise. */
-bool
-Error::OK() const
-{
+bool Error::OK() const {
     // The object is empty - users cannot make these, but this class can!
-    bool empty = ((_error_code == undefined_error)
-                  && (_error_message.empty()));
+    bool empty = ((_error_code == undefined_error) && (_error_message.empty()));
 
     // Just a message - the program part is null.
-    bool message = ((_error_code != undefined_error)
-                    && (!_error_message.empty()));
+    bool message = ((_error_code != undefined_error) && (!_error_message.empty()));
 
     DBG(cerr << "empty: " << empty << ", message: " << message << endl);
     return empty || message;
@@ -119,9 +103,7 @@ Error::OK() const
     @brief Parse an Error object.
     @param fp A valid file pointer to an input stream.
     @return TRUE if no error was detected, FALSE otherwise.  */
-bool
-Error::parse(FILE *fp)
-{
+bool Error::parse(FILE *fp) {
     if (!fp)
         throw InternalErr(__FILE__, __LINE__, "Null input stream");
 
@@ -134,8 +116,7 @@ Error::parse(FILE *fp)
     try {
         status = Errorparse(&arg) == 0;
         Error_delete_buffer(buffer);
-    }
-    catch (Error &e) {
+    } catch (Error &e) {
         Error_delete_buffer(buffer);
         throw InternalErr(__FILE__, __LINE__, e.get_error_message());
     }
@@ -147,10 +128,9 @@ Error::parse(FILE *fp)
     if (!status || !arg.status())
         throw InternalErr(__FILE__, __LINE__, "Error parsing error object!");
     else
-        return OK();  // Check object consistency
+        return OK(); // Check object consistency
 }
 
-
 /** Creates a printable representation of the Error object. It is suitable
     for framing, and also for printing and sending over a network.
 
@@ -161,23 +141,21 @@ Error::parse(FILE *fp)
 
     @param out A pointer to the output stream on which the Error object is to
     be rendered. */
-void
-Error::print(FILE *out) const
-{
+void Error::print(FILE *out) const {
     assert(OK());
 
-    fprintf(out, "Error {\n") ;
+    fprintf(out, "Error {\n");
 
-    fprintf(out, "    code = %d;\n", static_cast<int>(_error_code)) ;
+    fprintf(out, "    code = %d;\n", static_cast<int>(_error_code));
 
     // If the error message is wrapped in double quotes, print it, else, add
     // wrapping double quotes.
     if (*_error_message.begin() == '"' && *(_error_message.end() - 1) == '"')
-        fprintf(out, "    message = %s;\n", _error_message.c_str()) ;
+        fprintf(out, "    message = %s;\n", _error_message.c_str());
     else
-        fprintf(out, "    message = \"%s\";\n", _error_message.c_str()) ;
+        fprintf(out, "    message = \"%s\";\n", _error_message.c_str());
 
-    fprintf(out, "};\n") ;
+    fprintf(out, "};\n");
 }
 
 /** Creates a printable representation of the Error object. It is suitable
@@ -190,29 +168,25 @@ Error::print(FILE *out) const
 
     @param strm A reference to the output stream on which the Error object is to
     be rendered. */
-void
-Error::print(ostream &strm) const
-{
+void Error::print(ostream &strm) const {
     assert(OK());
 
-    strm << "Error {\n" ;
+    strm << "Error {\n";
 
-    strm << "    code = " << static_cast<int>(_error_code) << ";\n" ;
+    strm << "    code = " << static_cast<int>(_error_code) << ";\n";
 
     // If the error message is wrapped in double quotes, print it, else, add
     // wrapping double quotes.
     if (*_error_message.begin() == '"' && *(_error_message.end() - 1) == '"')
-        strm << "    message = " << _error_message.c_str() << ";\n" ;
+        strm << "    message = " << _error_message.c_str() << ";\n";
     else
-        strm << "    message = \"" << _error_message.c_str() << "\";\n"  ;
+        strm << "    message = \"" << _error_message.c_str() << "\";\n";
 
-    strm << "};\n" ;
+    strm << "};\n";
 }
 
 /** Get the ErrorCode for this instance. */
-ErrorCode
-Error::get_error_code() const
-{
+ErrorCode Error::get_error_code() const {
     assert(OK());
     return _error_code;
 }
@@ -223,35 +197,25 @@ Error::get_error_code() const
     for an OPeNDAP ErrorCode, the error message is left unchanged.
 
     @param ec The new ErrorCode value. */
-void
-Error::set_error_code(ErrorCode ec)
-{
+void Error::set_error_code(ErrorCode ec) {
     _error_code = ec;
     // Added check to make sure that err_messages is not accessed beyond its
     // bounds. 02/02/04 jhrg
-    if (_error_message.empty()
-        && ec > undefined_error && ec <= cannot_read_file) {
+    if (_error_message.empty() && ec > undefined_error && ec <= cannot_read_file) {
         _error_message = err_messages[ec - undefined_error];
-    }
-    else {
+    } else {
         _error_message = err_messages[0];
     }
 }
 
 /** Return the current error message. */
-string
-Error::get_error_message() const
-{
+string Error::get_error_message() const {
     assert(OK());
 
     return {_error_message};
 }
 
 /** Set the error message. */
-void
-Error::set_error_message(string msg)
-{
-    _error_message = std::move(msg);
-}
+void Error::set_error_message(string msg) { _error_message = std::move(msg); }
 
 } // namespace libdap
