@@ -42,22 +42,22 @@
 
 #ifndef WIN32
 #else
-#include <io.h>
 #include <fcntl.h>
+#include <io.h>
 #include <process.h>
 #endif
 
-//#define DODS_DEBUG
+// #define DODS_DEBUG
 
-#include "util.h"
 #include "debug.h"
+#include "util.h"
 
 #include "TestInt8.h"
 
 #include "TestByte.h"
 #include "TestInt16.h"
-#include "TestUInt16.h"
 #include "TestInt32.h"
+#include "TestUInt16.h"
 #include "TestUInt32.h"
 
 #include "TestInt64.h"
@@ -78,43 +78,24 @@ using std::endl;
 
 extern int test_variable_sleep_interval;
 
-void TestArray::_duplicate(const TestArray &ts)
-{
-    d_series_values = ts.d_series_values;
-}
+void TestArray::_duplicate(const TestArray &ts) { d_series_values = ts.d_series_values; }
 
-BaseType *
-TestArray::ptr_duplicate()
-{
-    return new TestArray(*this);
-}
+BaseType *TestArray::ptr_duplicate() { return new TestArray(*this); }
 
-TestArray::TestArray(const string &n, BaseType *v, bool is_dap4) :
-        Array(n, v, is_dap4), d_series_values(false)
-{
-}
+TestArray::TestArray(const string &n, BaseType *v, bool is_dap4) : Array(n, v, is_dap4), d_series_values(false) {}
 
-TestArray::TestArray(const string &n, const string &d, BaseType *v, bool is_dap4) :
-        Array(n, d, v, is_dap4), d_series_values(false)
-{
-}
+TestArray::TestArray(const string &n, const string &d, BaseType *v, bool is_dap4)
+    : Array(n, d, v, is_dap4), d_series_values(false) {}
 
-TestArray::TestArray(const TestArray &rhs) :
-        Array(rhs), TestCommon(rhs)
-{
-    _duplicate(rhs);
-}
+TestArray::TestArray(const TestArray &rhs) : Array(rhs), TestCommon(rhs) { _duplicate(rhs); }
 
-TestArray::~TestArray()
-{
-}
+TestArray::~TestArray() {}
 
-TestArray &
-TestArray::operator=(const TestArray &rhs)
-{
-    if (this == &rhs) return *this;
+TestArray &TestArray::operator=(const TestArray &rhs) {
+    if (this == &rhs)
+        return *this;
 
-    dynamic_cast<Array &>(*this) = rhs;
+    Array::operator=(rhs);
 
     _duplicate(rhs);
 
@@ -129,26 +110,24 @@ TestArray::operator=(const TestArray &rhs)
 // whole DDS is still present but only variables selected in the CE have
 // values.
 
-unsigned int TestArray::m_print_array(ostream &out, unsigned int index, unsigned int dims, unsigned int shape[])
-{
+unsigned int TestArray::m_print_array(ostream &out, unsigned int index, unsigned int dims, unsigned int shape[]) {
     if (dims == 1) {
         out << "{";
 
         // Added this test for zero-length arrays. jhrg 1/28/16
         if (shape[0] >= 1) {
             for (unsigned i = 0; i < shape[0] - 1; ++i) {
-                dynamic_cast<TestCommon&>(*var(index++)).output_values(out);
+                dynamic_cast<TestCommon &>(*var(index++)).output_values(out);
                 out << ", ";
             }
 
-            dynamic_cast<TestCommon&>(*var(index++)).output_values(out);
+            dynamic_cast<TestCommon &>(*var(index++)).output_values(out);
         }
 
         out << "}";
 
         return index;
-    }
-    else {
+    } else {
         out << "{";
         // Fixed an off-by-one error in the following loop. Since the array
         // length is shape[dims-1]-1 *and* since we want one less dimension
@@ -175,67 +154,58 @@ unsigned int TestArray::m_print_array(ostream &out, unsigned int index, unsigned
     }
 }
 
-void TestArray::output_values(std::ostream &out)
-{
-    //unsigned int *shape = new unsigned int[dimensions(true)];
+void TestArray::output_values(std::ostream &out) {
+    // unsigned int *shape = new unsigned int[dimensions(true)];
 
     vector<unsigned int> shape(dimensions(true));
     unsigned int index = 0;
     for (Dim_iter i = dim_begin(); i != dim_end() && index < dimensions(true); ++i)
         shape[index++] = dimension_size(i, true);
 
-    m_print_array(out, 0, dimensions(true), &shape[0]);
+    m_print_array(out, 0, dimensions(true), shape.data());
 
-    //delete[] shape;
-    //shape = 0;
+    // delete[] shape;
+    // shape = 0;
 }
 
 /** Special names are ones that start with 'lat' or 'lon'. These indicate
  that the vector (this is only for vectors) is a vector of latitude or
  longitude values. */
-bool TestArray::m_name_is_special()
-{
+bool TestArray::m_name_is_special() {
     return (name().find("lat") != string::npos || name().find("lon") != string::npos);
 }
 
-void TestArray::m_build_special_values()
-{
+void TestArray::m_build_special_values() {
     if (name().find("lat_reversed") != string::npos) {
         int array_len = length();
-        //double *lat_data = new double[array_len];
+        // double *lat_data = new double[array_len];
         vector<double> lat_data(array_len);
         for (int i = 0; i < array_len; ++i) {
             lat_data[i] = -89 + (180 / array_len) * (i + 1);
         }
-        libdap::set_array_using_double(this, &lat_data[0], array_len);
-    }
-    else if (name().find("lat") != string::npos) {
+        libdap::set_array_using_double(this, lat_data.data(), array_len);
+    } else if (name().find("lat") != string::npos) {
         int array_len = length();
         // double *lat_data = new double[array_len];
         vector<double> lat_data(array_len);
         for (int i = 0; i < array_len; ++i) {
             lat_data[i] = 90 - (180 / array_len) * (i + 1);
         }
-        libdap::set_array_using_double(this, &lat_data[0], array_len);
-    }
-    else if (name().find("lon") != string::npos) {
+        libdap::set_array_using_double(this, lat_data.data(), array_len);
+    } else if (name().find("lon") != string::npos) {
         int array_len = length();
-        //double *lon_data = new double[array_len];
+        // double *lon_data = new double[array_len];
         vector<double> lon_data(array_len);
         for (int i = 0; i < array_len; ++i) {
             lon_data[i] = (360 / array_len) * (i + 1);
         }
-        libdap::set_array_using_double(this, &lon_data[0], array_len);
-    }
-    else {
+        libdap::set_array_using_double(this, lon_data.data(), array_len);
+    } else {
         throw InternalErr(__FILE__, __LINE__, "Unrecognized name");
     }
 }
 
-int TestArray::m_offset(int y, Dim_iter X, int x)
-{
-    return y * dimension_size(X, false) + x;
-}
+int TestArray::m_offset(int y, Dim_iter X, int x) { return y * dimension_size(X, false) + x; }
 
 /**
  * @brief Load an 2D array with values.
@@ -246,9 +216,7 @@ int TestArray::m_offset(int y, Dim_iter X, int x)
  *
  * @param constrained_array
  */
-template<typename T, class C>
-void TestArray::m_constrained_matrix(vector<T>&constrained_array)
-{
+template <typename T, class C> void TestArray::m_constrained_matrix(vector<T> &constrained_array) {
     int unconstrained_size = 1;
     Dim_iter d = dim_begin();
     while (d != dim_end())
@@ -263,18 +231,22 @@ void TestArray::m_constrained_matrix(vector<T>&constrained_array)
         static_cast<C*>(var())->value(&v);
         else
 #endif
-        v = static_cast<C*>(var())->value();
+        v = static_cast<C *>(var())->value();
 
         whole_array[i] = v;
         var()->set_read_p(false); // pick up the next value
     }
 
-    DBG(cerr << "whole_array: "; copy(whole_array.begin(), whole_array.end(), ostream_iterator<T>(cerr, ", ")); cerr << endl);
+    DBG(cerr << "whole_array: "; copy(whole_array.begin(), whole_array.end(), ostream_iterator<T>(cerr, ", "));
+        cerr << endl);
 
     Dim_iter Y = dim_begin();
     Dim_iter X = Y + 1;
 
-    DBG(cerr << "dimension_start(Y): " << dimension_start(Y) << endl); DBG(cerr << "dimension_stop(Y): " << dimension_stop(Y) << endl); DBG(cerr << "dimension_start(X): " << dimension_start(X) << endl); DBG(cerr << "dimension_stop(X): " << dimension_stop(X) << endl);
+    DBG(cerr << "dimension_start(Y): " << dimension_start(Y) << endl);
+    DBG(cerr << "dimension_stop(Y): " << dimension_stop(Y) << endl);
+    DBG(cerr << "dimension_start(X): " << dimension_start(X) << endl);
+    DBG(cerr << "dimension_stop(X): " << dimension_stop(X) << endl);
 
     int constrained_size = 0;
     int y = dimension_start(Y);
@@ -290,9 +262,7 @@ void TestArray::m_constrained_matrix(vector<T>&constrained_array)
     }
 }
 
-template<typename T>
-void TestArray::m_enum_constrained_matrix(vector<T>&constrained_array)
-{
+template <typename T> void TestArray::m_enum_constrained_matrix(vector<T> &constrained_array) {
     int unconstrained_size = 1;
     Dim_iter d = dim_begin();
     while (d != dim_end())
@@ -302,17 +272,21 @@ void TestArray::m_enum_constrained_matrix(vector<T>&constrained_array)
     for (int i = 0; i < unconstrained_size; ++i) {
         T v;
         var()->read();
-        static_cast<D4Enum*>(var())->value(&v);
+        static_cast<D4Enum *>(var())->value(&v);
         whole_array[i] = v;
         var()->set_read_p(false); // pick up the next value
     }
 
-    DBG(cerr << "whole_array: "; copy(whole_array.begin(), whole_array.end(), ostream_iterator<T>(cerr, ", ")); cerr << endl);
+    DBG(cerr << "whole_array: "; copy(whole_array.begin(), whole_array.end(), ostream_iterator<T>(cerr, ", "));
+        cerr << endl);
 
     Dim_iter Y = dim_begin();
     Dim_iter X = Y + 1;
 
-    DBG(cerr << "dimension_start(Y): " << dimension_start(Y) << endl); DBG(cerr << "dimension_stop(Y): " << dimension_stop(Y) << endl); DBG(cerr << "dimension_start(X): " << dimension_start(X) << endl); DBG(cerr << "dimension_stop(X): " << dimension_stop(X) << endl);
+    DBG(cerr << "dimension_start(Y): " << dimension_start(Y) << endl);
+    DBG(cerr << "dimension_stop(Y): " << dimension_stop(Y) << endl);
+    DBG(cerr << "dimension_start(X): " << dimension_start(X) << endl);
+    DBG(cerr << "dimension_stop(X): " << dimension_stop(X) << endl);
 
     int constrained_size = 0;
     int y = dimension_start(Y);
@@ -332,40 +306,43 @@ void TestArray::m_enum_constrained_matrix(vector<T>&constrained_array)
  * Load the variable's internal data buffer with values, simulating a read()
  * call to some data store. A private method.
  */
-template<typename T, class C>
-void TestArray::m_cardinal_type_read_helper()
-{
+template <typename T, class C> void TestArray::m_cardinal_type_read_helper() {
     if (get_series_values()) {
         // Special case code for vectors that have specific names.
         // This is used to test code that works with lat/lon data.
         if (dimensions() == 1 && m_name_is_special()) {
             m_build_special_values();
-        }
-        else if (dimensions() == 2) {
-            vector<T> tmp(length());
+        } else if (dimensions() == 2) {
+            vector<T> tmp(length_ll());
             m_constrained_matrix<T, C>(tmp);
-            set_value(tmp, length());
-        }
-        else {
-            vector<T> tmp(length());
+            set_value_ll((T *)tmp.data(), length_ll());
+        } else {
+            vector<T> tmp(length_ll());
             for (int64_t i = 0, end = length(); i < end; ++i) {
                 var()->read();
-                tmp[i] = static_cast<C*>(var())->value();
+                tmp[i] = static_cast<C *>(var())->value();
                 var()->set_read_p(false); // pick up the next value
             }
-            set_value(tmp, length());
+            set_value_ll((T *)tmp.data(), length_ll());
         }
-    }
-    else {
+    } else {
         // read a value into the Array's prototype element
         var()->read();
-        T value = static_cast<C*>(var())->value();
-        vector<T> tmp(length());
-        for (int64_t i = 0, end = length(); i < end; ++i) {
+        T value = static_cast<C *>(var())->value();
+        vector<T> tmp(length_ll());
+        for (int64_t i = 0, end = length_ll(); i < end; ++i) {
             tmp[i] = value;
         }
 
-        set_value(tmp, length());
+#if 0
+        set_value(tmp, length_ll());
+#endif
+        // The following code inside #if 0 block always goes to set_value.
+        // May check why in the future. KY 2023-01-12
+#if 0
+        set_value_ll(tmp, length_ll());
+#endif
+        set_value_ll((T *)tmp.data(), length_ll());
     }
 }
 
@@ -373,33 +350,29 @@ void TestArray::m_cardinal_type_read_helper()
  * Load the variable's internal data buffer with values, simulating a read()
  * call to some data store. A private method.
  */
-template<typename T>
-void TestArray::m_enum_type_read_helper()
-{
+template <typename T> void TestArray::m_enum_type_read_helper() {
     if (get_series_values()) {
         if (dimensions() == 2) {
             vector<T> tmp(length());
             m_enum_constrained_matrix<T>(tmp);
             set_value(tmp, length());
-        }
-        else {
+        } else {
             vector<T> tmp(length());
             for (int64_t i = 0, end = length(); i < end; ++i) {
                 var()->read();
                 T v;
-                static_cast<D4Enum*>(var())->value(&v);
+                static_cast<D4Enum *>(var())->value(&v);
 
                 tmp[i] = v;
                 var()->set_read_p(false); // pick up the next value
             }
             set_value(tmp, length());
         }
-    }
-    else {
+    } else {
         // read a value into the Array's prototype element
         var()->read();
         T value;
-        static_cast<D4Enum*>(var())->value(&value);
+        static_cast<D4Enum *>(var())->value(&value);
 
         vector<T> tmp(length());
         for (int64_t i = 0, end = length(); i < end; ++i) {
@@ -410,19 +383,20 @@ void TestArray::m_enum_type_read_helper()
     }
 }
 
-bool TestArray::read()
-{
-    if (read_p()) return true;
+bool TestArray::read() {
+    if (read_p())
+        return true;
 
-    if (test_variable_sleep_interval > 0) sleep(test_variable_sleep_interval);
+    if (test_variable_sleep_interval > 0)
+        sleep(test_variable_sleep_interval);
 
-    int64_t array_len = length(); // elements in the array
+    int64_t array_len = length_ll(); // elements in the array
 
     switch (var()->type()) {
     // These are the DAP2 types and the classes that implement them all define
     // the old buf2val() and val2buf() methods. For the new DAP4 types see below.
-    //case dods_byte_c:
-    //case dods_uint8_c:
+    // case dods_byte_c:
+    // case dods_uint8_c:
     case dods_int16_c:
         m_cardinal_type_read_helper<dods_int16, Int16>();
         set_read_p(true);
@@ -476,7 +450,7 @@ bool TestArray::read()
         break;
 
     case dods_enum_c:
-        switch (static_cast<D4Enum*>(var())->element_type()) {
+        switch (static_cast<D4Enum *>(var())->element_type()) {
         case dods_byte_c:
         case dods_char_c:
         case dods_uint8_c:
@@ -517,13 +491,12 @@ bool TestArray::read()
             for (int64_t i = 0; i < array_len; ++i) {
                 var()->read();
                 // URL isa Str
-                tmp[i] = static_cast<Str*>(var())->value();
+                tmp[i] = static_cast<Str *>(var())->value();
                 var()->set_read_p(false); // pick up the next value
             }
-        }
-        else {
+        } else {
             var()->read();
-            string value = static_cast<Str*>(var())->value();
+            string value = static_cast<Str *>(var())->value();
 
             for (unsigned i = 0; i < array_len; ++i)
                 tmp[i] = value;
@@ -536,26 +509,26 @@ bool TestArray::read()
 
     case dods_opaque_c:
     case dods_structure_c:
+        vec_resize(array_len);
         for (unsigned i = 0; i < array_len; ++i) {
             // Copy the prototype and read a value into it
             BaseType *elem = var()->ptr_duplicate();
             elem->read();
             // Load the new value into this object's array
-            set_vec(i, elem);
+            set_vec_nocopy(i, elem); // Use set_vec_nocopy() TODO (and below)
         }
         set_read_p(true);
         break;
 
     case dods_sequence_c:
         // No sequence arrays in DAP2
-        if (!is_dap4()) throw InternalErr(__FILE__, __LINE__, "Bad data type");
+        if (!is_dap4())
+            throw InternalErr(__FILE__, __LINE__, "Bad data type");
 
+        vec_resize(array_len);
         for (unsigned i = 0; i < array_len; ++i) {
-            // Copy the prototype and read a value into it
-            BaseType *elem = var()->ptr_duplicate();
-            //elem->read();
-            // Load the new value into this object's array
-            set_vec(i, elem);
+            // Load the new BaseType (a D4Sequence) into the array element
+            set_vec_nocopy(i, var()->ptr_duplicate());
         }
 
         break;
@@ -566,7 +539,6 @@ bool TestArray::read()
     case dods_null_c:
     default:
         throw InternalErr(__FILE__, __LINE__, "Bad data type");
-        break;
     }
 
     // set_read_p(true);
@@ -574,8 +546,7 @@ bool TestArray::read()
     return true;
 }
 
-void TestArray::set_series_values(bool sv)
-{
-    dynamic_cast<TestCommon&>(*var()).set_series_values(sv);
+void TestArray::set_series_values(bool sv) {
+    dynamic_cast<TestCommon &>(*var()).set_series_values(sv);
     d_series_values = sv;
 }
