@@ -33,97 +33,74 @@
 //
 // jhrg 1/12/95
 
-
 #include "config.h"
 
+// #define DODS_DEBUG
+
 #ifndef WIN32
 #include <unistd.h>
 #else
-#include <io.h>
 #include <fcntl.h>
+#include <io.h>
 #include <process.h>
 #endif
 
-//#define DODS_DEBUG
-
 #include "TestInt32.h"
 #include "debug.h"
 
 extern int test_variable_sleep_interval;
 
-void
-TestInt32::_duplicate(const TestInt32 &ts)
-{
-    d_series_values = ts.d_series_values;
-}
+void TestInt32::_duplicate(const TestInt32 &ts) { d_series_values = ts.d_series_values; }
 
-TestInt32::TestInt32(const string &n) : Int32(n), d_series_values(false)
-{
-    d_buf = 1;
-}
+TestInt32::TestInt32(const string &n) : Int32(n), d_series_values(false) { d_buf = 1; }
 
-TestInt32::TestInt32(const string &n, const string &)
-    : Int32(n), d_series_values(false)
-{
-    d_buf = 1;
-}
+TestInt32::TestInt32(const string &n, const string &) : Int32(n), d_series_values(false) { d_buf = 1; }
 
-TestInt32::TestInt32(const TestInt32 &rhs) : Int32(rhs), TestCommon(rhs)
-{
-    _duplicate(rhs);
-}
+TestInt32::TestInt32(const TestInt32 &rhs) : Int32(rhs), TestCommon(rhs) { _duplicate(rhs); }
 
-TestInt32 &
-TestInt32::operator=(const TestInt32 &rhs)
-{
+TestInt32 &TestInt32::operator=(const TestInt32 &rhs) {
     if (this == &rhs)
-	return *this;
+        return *this;
 
-    dynamic_cast<Int32 &>(*this) = rhs; // run Constructor=
+    Int32::operator=(rhs); // run Constructor=
 
     _duplicate(rhs);
 
     return *this;
 }
 
-BaseType *
-TestInt32::ptr_duplicate()
-{
-    return new TestInt32(*this);
-}
+BaseType *TestInt32::ptr_duplicate() { return new TestInt32(*this); }
 
-void
-TestInt32::output_values(std::ostream &out)
-{
-    print_val(out, "", false);
-}
+void TestInt32::output_values(std::ostream &out) { print_val(out, "", false); }
 
-bool
-TestInt32::read()
-{
-    DBG(cerr << "Entering TestInt32::read for " << name() << endl);
+bool TestInt32::read() {
     if (read_p())
-	return true;
+        return true;
 
     if (test_variable_sleep_interval > 0)
-	sleep(test_variable_sleep_interval);
+        sleep(test_variable_sleep_interval);
 
     if (get_series_values()) {
-		// This line stopped working when I upgraded the compiler on osx 10.9.
-		// to version Apple LLVM version 5.1 (clang-503.0.38) (based on LLVM 3.4svn)
-		// jhrg 3/12/14
-		// d_buf = d_buf * 32;
-		d_buf <<= 5;
-		if (!d_buf)
-			d_buf = 32;
-    }
-    else {
+
+        // I added this in order to quell complaints from ASAN vis-a-vis
+        // runtime error: left shift of 1073741824 by 5 places cannot be represented in type 'dods_int32' (aka 'int')
+        // ndp 05/23/24
+        d_buf &= 0x07FFFFFF;
+
+        // This line stopped working when I upgraded the compiler on osx 10.9.
+        // to version Apple LLVM version 5.1 (clang-503.0.38) (based on LLVM 3.4svn)
+        // jhrg 3/12/14
+        // d_buf = d_buf * 32;
+        d_buf <<= 5;
+        if (!d_buf)
+            d_buf = 32;
+
+        DBGN(cerr << __PRETTY_FUNCTION__ << "d_buf: " << d_buf << endl);
+    } else {
         d_buf = 123456789;
     }
 
     set_read_p(true);
 
-    DBG(cerr << "In TestInt32::read, _buf = " << d_buf << endl);
-
     return true;
 }
