@@ -25,12 +25,9 @@
 
 #include <string>
 
-using std::string;
+namespace libdap {
 
-namespace libdap
-{
-
-time_t parse_time(const char * str, bool expand = true);
-string date_time_str(time_t *calendar, bool local = false);
+time_t parse_time(const char *str, bool expand = true);
+std::string date_time_str(time_t *calendar, bool local = false);
 
 } // namespace libdap
